/*
This program samples a voltage signal. The voltage signal 
represents the current vs time function in an ignition coil on 
which is applied 13v.  The voltage is measured on a 0.1 ohm 
resistor.  Therefore .1v represents 1 amp.  When a low-going
sync signal is detected, the timing starts for the sampling cycle.
Each sample's time is measured from the sync signal.

The output of the sampler is raw text cordinates for the result 
of each sample. The output gives the time of the sample, then the 
voltage of the sample, as a pair of data for each sample in a cycle.
One sample cycle has 420 samples, so 420 data pairs will be output
for one cycle.

The input signal will go to analog input A1 for analog sampling, 
and analog input A5 used as a digital input for synchronizing 
the sampling cycle.  When the A5 signal goes low, sampling 
will begin for 420 samples.  There will be 420 time and 
voltage readings taken for graphing in qbasic.  Qbasic will 
do the compensation math on the plot points.

The time data is in microseconds. The voltage data will range from 
0 to 1023, where 0 is 0 volts, and 1023 is 1.1 volts. (This is 
because the "INTERNAL" reference will be used, and it is 1.1v.)

The text will be output in cycles as raw data. It will be captured 
using Hyperterminal in Windows. The sampling cycle will start with 
the letter "$" and end with the letter "&".  The time of each sample 
will start with a "t" and each voltage sample will start with a "v".
Each piece of data will be separated by commas.

This program will use a much faster sampling time than normal. 
Instead of each analog reading taking 100usec, it will take about
20usec. The sample rate will be between 40k and 50k hz. Knowing 
the exact rate is not necessary since each sample will have its
time recorded.

However, since some coils are much slower, there will be an option
to have the last 100 samples taken at a much slower rate. This will
be done with a simple delay between samples.  Since the 
unsigned int can only take from 1 to 65,000, the max range of 
sampling is 65 msec. 

Tests show the time and voltage measurements are accurate enough
without doing any compensation for offset or magnitude. 
*/

unsigned long StartTime; // start time will be the time when the sampling starts (in microseconds) 
unsigned long Time;     // time will be in microseconds. 
unsigned int T[420];          // this is the array holding the time of each sample
unsigned int V[420];          // this is the array holding the amp value for each sample
unsigned int TimeComp = 12;   // this compensates for the time delay in analog read and time capture  
//this next part is used to increase sampling rate. 
//it gives about 9.5 msec of sampling.
//Remove it to sample at 1/5 the rate, but sample 5 times longer.
#define FASTADC 1
// defines for setting and clearing register bits
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif
  

void setup() {     
  
  //this section changes the sampling rate to about 45khz. 
  //it gives about 9.5 msec of sampling.
  //Remove it to sample at 1/5 the rate, but sample 5 times longer.
  #if FASTADC
  // set prescale to 16
  sbi(ADCSRA,ADPS2) ;
  cbi(ADCSRA,ADPS1) ;
  cbi(ADCSRA,ADPS0) ;
  #endif

  pinMode(A1, INPUT);        //this is the sampling input
  analogReference(INTERNAL);  //  internal is 1.1v)
  pinMode(A5, INPUT);        //this is the synchronizing input
  digitalWrite(A5, HIGH);    //set pullup for sync
  pinMode(13, OUTPUT);       //this is used as a transmit indicator LED
  Serial.begin(9600);       // begin transmitting data
}

void loop() {
  while (digitalRead(A5) == LOW) {}  //wait for sync signal to go high
  while (digitalRead(A5) == HIGH) {} //wait for sync signal to go low
  StartTime = micros() - TimeComp;   // start time of sampling cycle.  TimeComp compensates for the fact that there is a small delay in sampling, and that there are several lines of code before the first sample. The value is 16usec. This adds 16 usec to the time recorded for each sample. 4 to actually record the time of the sample. 4 for the delay in analogRead before the sample is taken. 4 to record the start time after the sync signal went low. And 4 for the setup of the for loop. The signal is now synced to what the oscope is showing.
  for (int x=0; x<=419; x++) {      //take 420 samples
    Time = micros() - StartTime;    //capture the time
    V[x] = analogRead(A1);         //save the sample value
    T[x] = int(Time);              //save the time of the sample
    if (x > 319) delayMicroseconds(500);           //Remove this line for normal sampling. This line will extend the time of the last 100 samples. 
  }
  digitalWrite(13, HIGH);           // LED on for transmit output
  Serial.println("");
  Serial.println(" $ ");            // print $ to signify start of sample cycle
  for (int y=0; y<=419; y++) {      //this loop outputs the data
    Serial.print("t");              // print t for time of sample
    Serial.print(T[y],DEC);         // print the time of the sample
    Serial.print(",v");             // print a for amps of sample
    Serial.print(V[y],DEC);         // print the amps of the sample
    Serial.print(",   ");           // . signifys end of one sample
  }
  Serial.println(" ");
  Serial.println(" & ");            //signifies end of sample cycle
  digitalWrite(13, LOW);            // LED off
  delay(1000); 
}
